from django.contrib.auth.models import User
from rest_framework import viewsets, permissions, serializers
from rest_framework.decorators import action
from rest_framework.response import Response

from .serializers import UserSerializer, UserWithDetailSerializer, UserLoginSerializer


class UserViewSet(viewsets.ModelViewSet):
    queryset = User.objects.prefetch_related('groups').all()
    serializer_class = UserSerializer

    @action(detail=True, methods=['post'])
    def add_permission(self, request, pk=None):
        from .services import user_add_permission
        permission_id = int(request.POST.get('permission_id'))
        user = user_add_permission(
            permission_id=permission_id,
            user_id=pk
        )
        serializer = self.get_serializer(user)
        return Response(serializer.data)

    @action(detail=True, methods=['post'])
    def add_group(self, request, pk=None):
        id_group = int(request.POST.get('id_group'))
        from .services import user_add_group
        user = user_add_group(
            grupo_id=id_group,
            user_id=pk
        )
        serializer = self.get_serializer(user)
        return Response(serializer.data)

    @action(detail=True, methods=['post'])
    def change_password(self, request, pk=None):
        from .services import user_change_password
        password_old = request.POST.get('password_old')
        password = request.POST.get('password')
        password_2 = request.POST.get('password_2')
        user_change_password(
            user_id=pk,
            password_new=password,
            password_new_confirmation=password_2,
            password_old=password_old
        )
        return Response({'result': 'La contraseña se ha cambiado correctamente'})

    @action(detail=False, methods=['get'], permission_classes=[permissions.IsAuthenticated])
    def validate_new_username(self, request) -> Response:
        validacion_reponse = {}
        from .services import user_exist_username
        username = self.request.GET.get('username', None)
        if user_exist_username(username=username):
            raise serializers.ValidationError({'username': 'Ya exite'})
        return Response(validacion_reponse)

    @action(detail=False, methods=['get'], permission_classes=[permissions.AllowAny])
    def validate_username_login(self, request) -> Response:
        from .services import user_exist_username
        validacion_reponse = {}
        username = self.request.GET.get('username', None)
        if not user_exist_username(username=username):
            raise serializers.ValidationError({'username': 'Este usuario no existe'})
        return Response(validacion_reponse)


class LoginViewSet(viewsets.ModelViewSet):
    serializer_class = UserLoginSerializer
    queryset = User.objects.all()

    @action(detail=False, methods=['post'], permission_classes=[permissions.AllowAny])
    def login(self, request) -> Response:
        from .services import user_get_token
        serializer = self.get_serializer(data=self.request.POST)
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data
        token = user_get_token(user_id=user.id)
        return Response({
            "user": UserWithDetailSerializer(user, context=self.get_serializer_context()).data,
            "token": token
        })

    @action(detail=False, methods=['get'], permission_classes=[permissions.AllowAny, ])
    def load_user(self, request) -> Response:
        if self.request.user.is_anonymous:
            serializer = UserWithDetailSerializer(None, context={'request': request})
            return Response(serializer.data)
        serializer = UserWithDetailSerializer(self.request.user, context={'request': request})
        return Response(serializer.data)



from django.contrib.auth import authenticate
from django.contrib.auth.models import User
from rest_framework import serializers
from .models import ExtendedUser


class UserSerializer(serializers.ModelSerializer):
    to_string = serializers.SerializerMethodField()
    profile_image_url = serializers.SerializerMethodField()
    security_pin = serializers.CharField(source='extended_user.security_pin', allow_null=True, required=False)
    security_qr = serializers.CharField(source='extended_user.security_qr', allow_null=True, required=False)
    gender = serializers.CharField(source='extended_user.gender', allow_null=True, required=False)
    date_of_birth = serializers.DateField(
        source='extended_user.date_of_birth', format="%Y-%m-%d",
        input_formats=['%Y-%m-%dT%H:%M:%S.%fZ', 'iso-8601']
    )

    def get_profile_image_url(self, obj):  # pragma: no cover
        return obj.extended_user.profile_image.url if hasattr(
            obj,
            'extended_user'
        ) and obj.extended_user.profile_image else None

    def get_to_string(self, instance):  # pragma: no cover
        return ('%s %s' % (instance.first_name, instance.last_name)).title()

    def update(self, instance, validated_data):
        extended_user = validated_data.pop('extended_user') if 'extended_user' in validated_data else None
        user = super().update(instance, validated_data)
        if extended_user is not None:
            security_pin = extended_user.get('security_pin', None)
            security_qr = extended_user.get('security_qr', None)
            gender = extended_user.get('gender', None)
            date_of_birth = extended_user.get('date_of_birth', None)
            if hasattr(user, 'extended_user'):
                extended_user = user.extended_user
            else:
                extended_user = ExtendedUser.objects.create(user_id=instance.id)
            extended_user.security_pin = security_pin
            extended_user.security_qr = security_qr
            extended_user.gender = gender
            extended_user.date_of_birth = date_of_birth
            extended_user.save()
        return user

    class Meta:
        model = User
        fields = [
            'id',
            'first_name',
            'last_name',
            'email',
            'is_active',
            'is_staff',
            'password',
            'username',
            'last_login',
            'date_joined',
            'is_superuser',
            'groups',
            'user_permissions',
            'to_string',
            'profile_image_url',
            'security_pin',
            'security_qr',
            'gender',
            'date_of_birth',
        ]
        extra_kwargs = {
            'password': {'write_only': True, 'required': False},
        }

    def create(self, validated_data):
        extended_user = validated_data.pop('extended_user') if 'extended_user' in validated_data else None
        validated_data.pop('groups', None)
        user = User.objects.create_user(**validated_data)
        user.set_password(validated_data['password'])
        user.save()
        if extended_user is not None:
            security_pin = extended_user.get('security_pin', None)
            security_qr = extended_user.get('security_qr', None)
            gender = extended_user.get('gender', None)
            date_of_birth = extended_user.get('date_of_birth', None)
            extended_user = ExtendedUser.objects.create(user_id=user.id)
            extended_user.security_pin = security_pin
            extended_user.security_qr = security_qr
            extended_user.gender = gender
            extended_user.date_of_birth = date_of_birth
            extended_user.save()
        return user


class UserWithDetailSerializer(UserSerializer):
    pass


class UserLoginSerializer(serializers.Serializer):
    username = serializers.CharField()
    password = serializers.CharField()

    def validate(self, data):
        user = authenticate(**data)
        if user and user.is_active:
            return user
        raise serializers.ValidationError({'_error': 'El usuario y la contraseña no coinciden con ningún usuario!'})



import base64
from django.conf import settings
from rest_framework import serializers
from Crypto.Cipher import AES


def encrypt(texto_plano: str) -> str:
    if not texto_plano:  # pragma: no cover
        return ''
    key = settings.KEY_ENCRY
    vector = settings.VECTOR_ENCRY
    if len(key) not in [16, 24, 32]:  # pragma: no cover
        raise serializers.ValidationError('La contraseña debe ser de 16,24 o 32 caracteres')
    if len(vector) != 16:  # pragma: no cover
        raise serializers.ValidationError('El vector debe tener 16 caracteres')
    obj = AES.new(key, AES.MODE_CFB, vector)
    ciphertext = base64.b64encode(obj.encrypt(texto_plano)).decode("utf-8")
    return ciphertext.strip()


def decrypt(enc: str) -> str:
    if not enc:  # pragma: no cover
        return ''
    key = settings.KEY_ENCRY
    vector = settings.VECTOR_ENCRY
    if len(key) not in [16, 24, 32]:  # pragma: no cover
        raise serializers.ValidationError('La contraseña debe ser de 16,24 o 32 caracteres')
    if len(vector) != 16:  # pragma: no cover
        raise serializers.ValidationError('El vector debe tener 16 caracteres')
    obj2 = AES.new(key, AES.MODE_CFB, vector)
    decrypted_text = obj2.decrypt(base64.b64decode(enc)).decode("utf-8")
    return decrypted_text



from rest_framework import serializers
from .models import GeneralInfoConfiguration


class GeneralInfoConfigurationSerializer(serializers.ModelSerializer):
    icon_small = serializers.ImageField(read_only=True)
    icon_medium = serializers.ImageField(read_only=True)
    logo_small = serializers.ImageField(read_only=True)
    logo_medium = serializers.ImageField(read_only=True)

    def update(self, instance, validated_data):
        logo = validated_data.get('logo', None)
        application_name = validated_data.get('application_name', None)
        from .services import create_update_general_info
        configuracion = create_update_general_info(logo=logo, application_name=application_name)
        return configuracion

    class Meta:
        model = GeneralInfoConfiguration
        fields = [
            'id',
            'logo',
            'application_name',
            'icon_small',
            'icon_medium',
            'logo_small',
            'logo_medium',
        ]



from django.contrib.auth.models import User, Permission, Group
from knox.models import AuthToken
from rest_framework import serializers


def user_get_token(
        user_id: int
) -> str:
    user = User.objects.get(pk=user_id)
    tokens = AuthToken.objects.filter(user=user)
    tokens.delete()
    _, token = AuthToken.objects.create(user)
    return token


def user_exist_username(
        username: str
) -> bool:
    if username and User.objects.filter(username=username).exists():
        return True
    return False


def user_change_password(
        user_id: int,
        password_old: str,
        password_new: str,
        password_new_confirmation: str
) -> User:
    user = User.objects.get(pk=user_id)
    if not user.check_password(password_old):
        raise serializers.ValidationError({'_error': 'La contraseña suministrada no coincide con el user'})
    if not password_new == password_new_confirmation:
        raise serializers.ValidationError({'_error': 'La contraseña nueva con su confirmación no coincide'})
    user.set_password(password_new)
    user.save()
    return user


def user_add_permission(
        permission_id: int,
        user_id: int
) -> User:
    user = User.objects.get(pk=user_id)
    permission = Permission.objects.get(id=permission_id)
    has_group = user.user_permissions.filter(id=permission_id).exists()
    if not has_group:
        user.user_permissions.add(permission)
    else:
        user.user_permissions.remove(permission)
    return user


def user_add_group(
        grupo_id: int,
        user_id: int
) -> User:
    user = User.objects.get(pk=user_id)
    group = Group.objects.get(id=grupo_id)

    has_group = user.groups.filter(id=grupo_id).exists()
    if not has_group:
        user.groups.add(group)
    else:
        user.groups.remove(group)
    return user
